class AggregateHelper:
    """
    Not finished yet.
    """

    def match(self, *args):
        pass

    def sort(self, *args):
        pass

    def group(self, *args):
        pass

    def project(self, *args):
        pass

    def set(self, *args):
        pass

    def max(self, *args):
        pass

    def min(self, *args):
        pass

    def last(self, *args):
        pass

    def first(self, *args):
        pass

    def avg(self, *args):
        pass

    def divide(self, *args):
        pass

    def substract(self, *args):
        pass

    def if_null(self, *args):
        pass
